#include <allegro.h>

#include <string>
#include <fstream>

#include "GameState.h"
#include "Game.h"
#include "Menu.h"

using namespace std;

Game::Game(): myScene(NULL), next(NULL), player(NULL), clickX(0), clickY(0), inClick(false), walkX(0), walkY(0), inWalk(false)
{
}

Game::~Game()
{
	delete myScene;
	delete player;
}

GameState *Game::RunLogic()
{
	if(myScene == NULL || player == NULL)
		return NULL;
	
	if(key[KEY_ESC])
	{
		if(SafeToSave())
		{
			switch(alert3("Save Game?", "", "Would you like to save your game?", "&Yes", "&No", "&Cancel", 'y', 'n', 'c'))
			{
			case 1: // save
				char fn[512];
				strncpy(fn, filename.c_str(), 511);
				fn[512] = '\0';
				if(file_select_ex("Save As?", fn, "dat", 510, 320, 240))
					if(!Save(fn))
						if(alert("Save Failed.", "", "Return to game or quit?", "&Return", "&Quit", 'r', 'q') == 1)
							break;
						else;
					else;
				else
					break; // return to game

			case 2:
				delete this;
				return new Menu();

			// "case 3"
			// otherwise, they hit cancel, keep playing
			}
		}
		else
			if(alert("Paused", "Game paused. Click Unpause to continue.", "Clicking Quit will NOT save your game.", "&Unpause", "&Quit", 'u', 'q') == 2)
			{
				delete this;
				return new Menu();
			}
	}
	
	int mx = mouse_x / (SCREEN_W / 320), my = mouse_y / (SCREEN_H / 240);
	if(mouse_b & 1 && !inClick)
	{
		inClick = true;
		clickX = mx;
		clickY = my;
	}
	if(!(mouse_b & 1) && inClick)
	{
		inClick = false;
		walkX = mx + myScene->panX - 320 / 2 - player->GetBitmap()->w / 2;
		walkY = my + myScene->panY - 240 / 2 + player->GetBitmap()->h / 2;
		if(walkX < 0)
			walkX = 0;
		if(walkX >= myScene->background->w)
			walkX = myScene->background->w - 1;
		if(walkY < 0)
			walkY = 0;
		if(walkY >= myScene->background->h)
			walkY = myScene->background->h - 1;
		inWalk = true;
	}
	
	if(inWalk)
	{
		int nx, ny;
		if(player->x > walkX)
			nx = player->x - 2;
		else if(player->x < walkX - 1)
			nx = player->x + 2;
		else
			nx = walkX;
		if(player->y > walkY)
			ny = player->y - 2;
		else if(player->y < walkY - 1)
			ny = player->y + 2;
		else
			ny = walkY;
		if(!myScene->MoveObject(player, nx, ny) || (nx == walkX && ny == walkY))
		{
			myScene->TryInteract(player->x + player->GetBitmap()->w / 2, player->y - player->GetBitmap()->h / 2);
			inWalk = false;
		}
	}
		
	myScene->Logic();
	
	if(next)
	{
		delete myScene;
		myScene = next;
		next = NULL;
	}

	return this;
}

void Game::Draw(BITMAP *dest)
{
	myScene->Draw(dest);
	int mx = mouse_x / (SCREEN_W / 320), my = mouse_y / (SCREEN_H / 240);
	hline(dest, mx - 6, my, mx - 3, makecol(255, 255, 255));
	hline(dest, mx + 3, my, mx + 6, makecol(255, 255, 255));
	vline(dest, mx, my - 6, my - 3, makecol(255, 255, 255));
	vline(dest, mx, my + 3, my + 6, makecol(255, 255, 255));
}

bool Game::SafeToSave()
{
	for(list<Object*>::iterator i = myScene->objects.begin(); i != myScene->objects.end(); i++)
	{
		for(std::vector<Event>::iterator j = (*i)->events.begin(); j != (*i)->events.end(); j++)
		{
			if(j->running)
				return false;
		}
	}
	
	for(std::vector<Event>::iterator i = myScene->events.begin(); i != myScene->events.end(); i++)
	{
		if(i->running)
			return false;
	}
	
	return true;
}

void Game::GoToScene(const string& filename)
{
	string::size_type pos = filename.find("#");
	if(pos == string::npos)
		return;
	
	string fn = filename.substr(0, pos);
	string entry = filename.substr(fn.length() + 1);

	next = new Scene(this);
	if(!next->Load(fn))
	{
		alert("Failed to load scene", fn.c_str(), NULL, "&Okay", NULL, 'o', 0);
		delete next;
		return;
	}
	next->FireEvent("load");
	
	next->SetPlayer(player);
	if(!next->MoveToEntry(player, entry))
	{
		alert("Undefined entry point", entry.c_str(), NULL, "&Okay", NULL, 'o', 0);
		return;
	}
	
	for(list<Object*>::iterator i = next->objects.begin(); i != next->objects.end(); i++)
	{
		(*i)->FireEvent("load");
	}
}

void Game::New()
{
	myScene = new Scene(this);
	if(!myScene->Load("media/scene01.dat"))
	{
		allegro_message("Failed to load media/scene01.dat");
		delete myScene;
		myScene = NULL;
	}
	myScene->FireEvent("load");
	
	player = new Character();
	player->scene = myScene;
	player->bmp = load_bitmap("media/mans.bmp", NULL);
	if(player->bmp == NULL)
	{
		allegro_message("Failed to load mans.");
		delete player;
		player = NULL;
	}
	else
	{
		myScene->SetPlayer(player);
		if(!myScene->MoveToEntry(player, "entry"))
			allegro_message("Undefined entry point");
	}
}

bool Game::Load(const std::string& location)
{
	PACKFILE* pf = pack_fopen(location.c_str(), "rp");
	if(!pf)
		return false;
		
	filename = location;
	string sceneFile;
	int saveX, saveY;
	
	char* buffer;
	int length;
	
	length = pack_igetw(pf);
	buffer = new char[length + 1];
	pack_fread(buffer, length, pf);
	buffer[length] = '\0';
	sceneFile = buffer;
	delete [] buffer;
	
	saveX = pack_igetw(pf);
	saveY = pack_igetw(pf);
	
	length = pack_igetw(pf);
	for(int i = 0; i < length; i++)
	{
		string var;
		
		int length2 = pack_igetw(pf);
		buffer = new char[length2 + 1];
		pack_fread(buffer, length2, pf);
		buffer[length2] = '\0';
		var = buffer;
		delete [] buffer;
		
		int val = pack_igetl(pf);
		gameVars[var] = val;
	}
	
	length = pack_igetw(pf);
	for(int i = 0; i < length; i++)
	{
		string key;
		map<string, int> temp;
		
		int length2 = pack_igetw(pf);
		buffer = new char[length2 + 1];
		pack_fread(buffer, length2, pf);
		buffer[length2] = '\0';
		key = buffer;
		delete [] buffer;
		
		length2 = pack_igetw(pf);
		for(int j = 0; j < length2; j++)
		{
			string var;
			
			int length3 = pack_igetw(pf);
			buffer = new char[length3 + 1];
			pack_fread(buffer, length3, pf);
			buffer[length3] = '\0';
			var = buffer;
			delete [] buffer;
			
			int val = pack_igetl(pf);
			temp[var] = val;
		}
		
		sceneVars[key] = temp;
	}
		
	pack_fclose(pf);
	
	myScene = new Scene(this);
	if(!myScene->Load(sceneFile))
	{
		alert("Failed to load scene", sceneFile.c_str(), NULL, "&Okay", NULL, 'o', 0);
		delete myScene;
		myScene = NULL;
		return false;
	}
	
	player = new Character();
	player->scene = myScene;
	player->bmp = load_bitmap("media/mans.bmp", NULL);
	if(player->bmp == NULL)
	{
		delete player;
		player = NULL;
	}
	else
	{
		myScene->SetPlayer(player);
		myScene->MoveObject(player, saveX, saveY);
	}
	
	return true;
}

bool Game::Save(const std::string& location)
{
#ifndef PACKFILE_COMPRESSION
	const char* mode = "w!";
#else
	const char* mode = "wp";
#endif
	PACKFILE* pf = pack_fopen(location.c_str(), mode);
	if(!pf)
		return false;
		
	filename = location;
	
	pack_iputw(myScene->title.length(), pf);
	pack_fwrite(myScene->title.c_str(), myScene->title.length(), pf);
	
	pack_iputw(player->x, pf);
	pack_iputw(player->y, pf);
	
	pack_iputw(gameVars.size(), pf);
	for(map<string, int>::iterator i = gameVars.begin(); i != gameVars.end(); i++)
	{
		pack_iputw(i->first.length(), pf);
		pack_fwrite(i->first.c_str(), i->first.length(), pf);
		pack_iputl(i->second, pf);
	}
	
	pack_iputw(sceneVars.size(), pf);
	for(map<string, map<string, int> >::iterator i = sceneVars.begin(); i != sceneVars.end(); i++)
	{
		pack_iputw(i->first.length(), pf);
		pack_fwrite(i->first.c_str(), i->first.length(), pf);
		pack_iputw(i->second.size(), pf);
		for(map<string, int>::iterator j = i->second.begin(); j != i->second.end(); j++)
		{
			pack_iputw(j->first.length(), pf);
			pack_fwrite(j->first.c_str(), j->first.length(), pf);
			pack_iputl(j->second, pf);
		}
	}
	
	pack_fclose(pf);
	return true;
}
