#ifndef OBJECT_H
#define OBJECT_H

#include <allegro.h>

#include <vector>

#include "Script.h"

class Scene;

struct Event
{
	Event(const Event& other): name(other.name), running(other.running), data(other.data)
	{ }
	Event(const std::string& name, const Script& data): name(name), running(false), data(data)
	{ }
	std::string name;
	bool running;
	Script data;
};

class Object
{
public:
	int x, y, z;
	int w, h;
	int give;
	/// Scene owns this object
	bool sceneOwned;
	/// Scene this object is in
	Scene* scene;
	std::vector<Event> events;
	/// Text balloon color
	int color;
	/// Text balloon content
	std::string message;
	/// Ready for deletion
	bool deleteMe;
	/// Is an obstacle
	bool obstacle;

	Object(): x(0), y(0), z(0), w(0), h(0), give(0), sceneOwned(false), scene(NULL), color(0), deleteMe(false), obstacle(false)
	{
	}
	
	virtual ~Object() = 0;
	
	virtual void Logic()
	{
		for(std::vector<Event>::iterator i = events.begin(); i != events.end(); i++)
		{
			if(i->running)
				i->running = i->data.Logic();
		}
	}
	
	bool FireEvent(const std::string& name);
	
	void DrawScripts(BITMAP* dest);

	virtual BITMAP *GetBitmap()
	{
		return NULL;
	}
};

#endif
