#ifndef SCENE_H
#define SCENE_H

#include <allegro.h>

#include <string>
#include <list>
#include <vector>

#include "Object.h"
#include "Character.h"
#include "Script.h"

class Game;

class Scene
{
public:
	struct Entry
	{
		std::string name;
		int x;
		int y;
		int z;
	};
	
public:
	std::string title;
	BITMAP* background;
	BITMAP* collision;
	Character* player;
	std::list<Object*> objects;
	std::vector<Entry*> entries;
	std::vector<Event> events;
	
	int panX, panY;
	Game* myGame;

public:
	Scene(Game* myGame);
	~Scene();

	bool Load(const std::string& fileName);
	bool Save(const std::string& location);
	
	void SetPlayer(Character* player);
	bool MoveToEntry(Object* object, const std::string& entry);
	bool MoveObject(Object* object, int x, int y);
	void SortObjects();
	bool FireEvent(const std::string& name);
	void TryInteract(int x, int y);
	void GoToScene(const std::string& filename);
	
	void SetGameVariable(const std::string& var, int val);
	int GetGameVariable(const std::string& var);
	void SetSceneVariable(const std::string& var, int val);
	int GetSceneVariable(const std::string& var);
	
	void Logic();
	void Draw(BITMAP *dest);
};

#endif
