#include <allegro.h>
#include <assert.h>

#include <string>
#include <list>
#include <iostream>
#include <fstream>
#include <sstream>

#include "Compiler.h"
#include "Script.h"
#include "ScriptData.h"

using namespace std;

Script::Script(): data(NULL), plane(NULL), context(NULL), saveContext(NULL), scene(NULL), sleeping(0)
{
}

Script::Script(const Script& other): data(NULL), plane(NULL), context(other.context), saveContext(other.saveContext), scene(other.scene), sleeping(0)
{
	if(other.plane != NULL)
	{
		plane = create_bitmap(other.plane->w, other.plane->h);
		assert(plane != NULL);
		blit(other.plane, plane, 0, 0, 0, 0, plane->w, plane->h);
	}
	if(other.data != NULL)
	{
		data = new ScriptData(this, *other.data);
	}
}

Script::~Script()
{
	delete data;
	destroy_bitmap(plane);
}

bool Script::CompileFile(const string& content, string& error)
{
	ifstream file(content.c_str(), ios::in);
	if(!file.good())
	{
		error = "Unable to open input file";
		return false;
	}
	
	return CompileInternal(file, error);
}

bool Script::Compile(const string& content, string& error)
{
	stringstream file(content);
	return CompileInternal(file, error);
}

void Script::Save(PACKFILE* pf)
{
	pack_iputl(data->codeE - data->code, pf);
	pack_fwrite(data->code, data->codeE - data->code, pf);
}

bool Script::Load(Object* context, PACKFILE* pf)
{
	SetContext(context);

	data = new ScriptData(this);
	int length = pack_igetl(pf);
	data->code = new unsigned char [length];
	data->codeP = data->code;
	data->codeE = data->code + length;
	pack_fread(data->code, length, pf);
	return true;
}

void Script::SetContext(Object* context)
{
	saveContext = this->context = context;
	plane = create_bitmap(320, 240);
	assert(plane != NULL);
	clear_to_color(plane, makecol(255, 0, 255));
}

void Script::SetScene(Scene* scene)
{
	this->scene = scene;
}

void Script::Reset()
{
	data->Reset();
	context = saveContext;
	sleeping = 0;
}

bool Script::Logic()
{
	assert(plane != NULL);
	if(sleeping > 0)
	{
		--sleeping;
		return true;
	}
	while(true)
	{
		if(!data->Step())
			return false;
		if(sleeping > 0)
			return true;
	}
}

bool Script::CompileInternal(istream& file, string& error)
{
	Compiler c(file);
	
	if(!c.Compile())
	{
		error = c.errStream.str();
		return false;
	}
	
	c.ShrinkCode();
	
	data = new ScriptData(this);
	int size = c.codeE - c.code;
	data->code = new unsigned char [size];
	data->codeP = data->code;
	data->codeE = data->code + size;
	memcpy(data->code, c.code, size);
	
	return true;
}
