#ifndef SCRIPTDATA_H
#define SCRIPTDATA_H

#include <assert.h>

#include <string>

#include "Opcodes.h"

class WeakType
{
public:
	enum Type {
		T_Null,
		T_String,
		T_Integer
	};
	
private:
	Type type;
	union {
		char* string;
		int integer;
	} data;
	
public:
	WeakType()
	{
		type = T_Null;
	}
	WeakType(const std::string& string)
	{
		type = T_String;
		data.string = new char [string.length() + 1];
		strcpy(data.string, string.c_str());
	}
	WeakType(int integer)
	{
		type = T_Integer;
		data.integer = integer;
	}
	WeakType(const WeakType& other)
	{
		*this = other;
	}
	
	WeakType& operator =(const WeakType& other)
	{
		type = other.type;
		switch(other.type)
		{
			case T_Null:
				break;
			case T_String:
				data.string = new char [strlen(other.data.string) + 1];
				strcpy(data.string, other.data.string);
				break;
			case T_Integer:
				data.integer = other.data.integer;
				break;
		}

		return *this;
	}
	
	~WeakType()
	{
		switch(type)
		{
			case T_Null:
				break;
			case T_String:
				delete [] data.string;
				break;
			case T_Integer:
				break;
		}
	}
	
	operator const char*()
	{
		assert(type == T_String);
		return data.string;
	}
	
	operator int()
	{
		assert(type == T_Integer);
		return data.integer;
	}
	
	Type GetType()
	{
		return type;
	}
};

class Script;

class ScriptData
{
	friend class Script;
private:
	typedef void (ScriptData::*Opcode)(WeakType& result);
	static Opcode opcodes[];
private:
	unsigned char* code, *codeP, *codeE;
	Script* script;
public:
	ScriptData(Script* script);
	ScriptData(Script* script, const ScriptData& other);
	~ScriptData();
	
	bool Step();
	void Reset();
	void Step(WeakType& result);
	
	void VoidConstant(WeakType& result);
	void IntegerConstant(WeakType& result);
	void StringConstant(WeakType& result);
	void SetContext(WeakType& result);
	void Jump(WeakType& result);
	void JumpZero(WeakType& result);
	void Rest(WeakType& result);
	void Color(WeakType& result);
	void Clear(WeakType& result);
	void ClearPrint(WeakType& result);
	void PrintCenter(WeakType& result);
	void Print(WeakType& result);
	void SetGameVariable(WeakType& result);
	void GetGameVariable(WeakType& result);
	void SetSceneVariable(WeakType& result);
	void GetSceneVariable(WeakType& result);
	void Destroy(WeakType& result);
	void Hide(WeakType& result);
	void Load(WeakType& result);
	
private:
	ScriptData(const ScriptData& other);
};

#endif
