#define ALLEGRO_NO_MAGIC_MAIN
#include <string>
#include <iostream>
#include <vector>
#include <fstream>
#include <allegro.h>
#include <assert.h>
#include "Scene.h"
#include "SceneryObject.h"

using namespace std;

bool load_scene(Scene& l)
{
	string filename;
	BITMAP* bitmap;
	bool hasErrors = false;
	
	filename = get_config_string("scene", "background", "");
	if(filename.length() == 0)
	{
		cerr << "Scene does not have a background." << endl;
		return false;
	}
	if((bitmap = load_bitmap(filename.c_str(), NULL)) == NULL)
	{
		cerr << "Failed to load background " << filename << endl;
		return false;
	}
	l.background = bitmap;
	
	filename = get_config_string("scene", "collision", "");
	if(filename.length() != 0)
	{
		if((bitmap = load_bitmap(filename.c_str(), NULL)) == NULL)
		{
			cerr << "Failed to load collision " << filename << endl;
			return false;
		}
	}
	else
	{
		bitmap = create_bitmap(l.background->w, l.background->h);
		rectfill(bitmap, 0, 0, bitmap->w - 1, bitmap->h - 1, -1);
	}
	l.collision = bitmap;
	
	const char** events = NULL;
	int numEvents = list_config_entries("scene", &events);
	for(int i = 0; i < numEvents; i++)
	{
		string event = events[i];
		if(event.substr(0, 6) == "event/")
		{
			string eventName = event.substr(6);
			string filename = get_config_string("scene", events[i], "");
			ifstream file(filename.c_str(), ios::in);
			if(!file.good())
			{
				cerr << "Unable to open script for " << eventName << ": " << filename << endl;
				hasErrors = true;
				continue;
			}
			string data;
			while(true)
			{
				char c = file.get();
				if(!file.good())
					break;
				data += c;
			}
			
			Script s;
			string error;
			if(!s.Compile(data, error))
			{
				cerr << "While compiilng " << filename << ":" << endl;
				cerr << error << endl;
				hasErrors = true;
				continue;
			}
			Event e(eventName, s);
			l.events.push_back(e);
		}
	}
	
	const char** sections = NULL;
	int numSections = list_config_sections(&sections);
	for(int i = 0; i < numSections; i++)
	{
		string section = sections[i];
		section = section.substr(1, section.length() - 2);
		if(section == "scene")
			continue;
		
		if(section.substr(0, 6) == "entry/")
		{
			string entryName = section.substr(6);
			
			int x = get_config_int(section.c_str(), "x", -1);
			if(x == -1)
			{
				cerr << "Entry " << entryName << " missing X position" << endl;
				hasErrors = true;
				continue;
			}
			
			int y = get_config_int(section.c_str(), "y", -1);
			if(y == -1)
			{
				cerr << "Entry " << entryName << " missing Y position" << endl;
				hasErrors = true;
				continue;
			}
			
			int z = get_config_int(section.c_str(), "z", 0);
			
			Scene::Entry* e = new Scene::Entry();
			e->name = entryName;
			e->x = x;
			e->y = y;
			e->z = z;
			l.entries.push_back(e);
		}
		else if(section.substr(0, 7) == "object/")
		{
			string objectName = section.substr(7);
			SceneryObject* o = new SceneryObject();
			
			filename = get_config_string(section.c_str(), "display", "");
			if(filename.length() != 0)
			{
				if((bitmap = load_bitmap(filename.c_str(), NULL)) == NULL)
				{
					cerr << "Failed to load " << section << "'s display " << filename << endl;
					hasErrors = true;
					continue;
				}
			}
			else
				bitmap = NULL;
			o->bmp = bitmap;
			
			o->x = get_config_int(section.c_str(), "x", -1);
			if(o->x == -1)
			{
				cerr << "Object " << objectName << " missing X position" << endl;
				hasErrors = true;
				continue;
			}
			
			o->y = get_config_int(section.c_str(), "y", -1);
			if(o->y == -1)
			{
				cerr << "Object " << objectName << " missing Y position" << endl;
				hasErrors = true;
				continue;
			}
			
			o->z = get_config_int(section.c_str(), "z", 0);
			
			o->w = get_config_int(section.c_str(), "w", -1);
			if(o->w == -1)
				o->w = o->bmp->w;
			
			o->h = get_config_int(section.c_str(), "h", -1);
			if(o->h == -1)
				o->h = o->bmp->h;
			
			o->give = get_config_int(section.c_str(), "give", 0);
			o->obstacle = get_config_int(section.c_str(), "obstacle", 0);
	
			events = NULL;
			numEvents = list_config_entries(section.c_str(), &events);
			for(int i = 0; i < numEvents; i++)
			{
				string event = events[i];
				if(event.substr(0, 6) == "event/")
				{
					string eventName = event.substr(6);
					string filename = get_config_string(section.c_str(), events[i], "");
					ifstream file(filename.c_str(), ios::in);
					if(!file.good())
					{
						cerr << "Unable to open script for " << eventName << ": " << filename << endl;
						hasErrors = true;
						continue;
					}
					string data;
					while(true)
					{
						char c = file.get();
						if(!file.good())
							break;
						data += c;
					}
					
					Script s;
					string error;
					if(!s.Compile(data, error))
					{
						cerr << "While compiilng " << filename << ":" << endl;
						cerr << error << endl;
						hasErrors = true;
						continue;
					}
					Event e(eventName, s);
					o->events.push_back(e);
				}
			}
			
			l.objects.push_back(o);
		}
		else
		{
			cerr << "Unknown configuration section " << section << endl;
			hasErrors = true;
			continue;
		}
	}
	
	return !hasErrors;
}

int main(int argc, char* argv[])
{
	install_allegro(SYSTEM_NONE, &errno, &atexit);
	
	set_color_depth(16);
	set_color_conversion(COLORCONV_TOTAL);
	
	if(argc < 2)
	{
		cerr << "Usage: " << argv[0] << " DIRECTORY" << endl;
		cerr << " DIRECTORY must have a scene.ini file." << endl;
		return 1;
	}
	
	char* cwd = new char [1024];
	cwd = getcwd(cwd, 1024);
	assert(cwd != NULL);
	
	if(chdir(argv[1]) != 0)
	{
		cerr << "Couldn't chdir to " << argv[1] << endl;
		return 1;
	}
	
	if(!file_exists("scene.ini", FA_ALL, NULL))
	{
		cerr << "scene.ini doesn't exist in " << argv[1] << "." << endl;
		return 1;
	}
	
	set_config_file("scene.ini");
	
	Scene* l = new Scene(NULL);
	
	if(!load_scene(*l))
		return 1;
		
	char* buffer = new char [strlen(argv[1]) + 5];
	strcpy(buffer, argv[1]);
	put_backslash(buffer);
	buffer[strlen(buffer) - 1] = '\0';
	strcat(buffer, ".dat");
	
	if(chdir(cwd) != 0)
	{
		cerr << "Couldn't chdir to " << cwd << endl;
		return 1;
	}
	
	if(!l->Save(buffer))
	{
		delete [] buffer;
		return 1;
	}
	
	delete [] buffer;
	
	delete l;
	
	delete [] cwd;
	
	allegro_exit();
	return 0;
}
